/*!
// @file		MessageComposer.hpp
//
// @brief		C++ Message composing example class
//
// @version		Mailit for C/C++ 1.0 sample file
//
// @author		(C) 2005 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#ifndef __MESSAGECOMPOSER_HPP__
#define __MESSAGECOMPOSER_HPP__

#include "EmaiMessage.hpp"
#include <vector>

/***************************************************/
//
// MessageComposer class
//
/***************************************************/

/*!
// @class	MessageComposer
// @brief	MessageComposer class
*/

class MessageComposer
{
private:

	Emai::MessagePart					mTextPart;
    Emai::MessagePart					mHtmlPart;
	
	typedef std::vector<Emai::MessagePart>
										MessagePartArray;

	MessagePartArray					mAttachments; 
    MessagePartArray					mInlines;

public:

										MessageComposer();

										~MessageComposer();

private:

	Emai::MessagePart					CreateAlternativeMultipart();

	Emai::MessagePart					CreateRelatedMultipart(
											EmaiUint32 parameterValueFlag);

	Emai::MessagePart					CreateMixedMultipart();

	void								BindAttachments(
											Emai::MessagePart& inMessagePart,
											const MessagePartArray& inAttachments);
	
public:

	/*!
	// @brief	Sets the plain text message part
	//
	// @param	inPlainText - pointer to the plain text data
	// @param	inCharacterSet - character set of the inPlainText data
	*/
	void								SetText(
											EmaiConstCharPtr inPlainText,
											EmaiUint32 inCharacterSet);

	/*!
	// @brief	Set the Unicode text message part
	//
	// @param	inUnicodeText - pointer to the unicode text data
	// @param	inCharacterSet - character set of the inPlainText data
	*/
	void								SetUnicodeText(
											EmaiConstUniCharPtr inUnicodeText);
	
	/*!
	// @brief	Set the HTML message part
	//
	// @param	inHtmlText - pointer to the html text data
	*/
	void								SetHtml(
											EmaiConstCharPtr inHtmlText);

	/*!
	// @brief	Add the attachment data to the list of the attachments
	//
	// @param	inData - pointer to the data
	// @param	inDataSize - size of the data
	// @param	inFileName - pointer to the name of the data
	*/
	void								AddAttachmentFromData(
											const void* inData,
											EmaiUint32 inDataSize,
											EmaiConstUniCharPtr inFileName);

	/*!
	// @brief	Add the attachment file to the list of the attachments
	//
	// @param	inFileName - pointer to the name of the file
	// @param	inFileFullPath - pointer to the full path of the file
	*/
	void								AddAttachmentFromFile(
											EmaiConstUniCharPtr inFileName,
											EmaiConstUniCharPtr inFileFullPath);

	/*!
	// @brief	Add the inline attachment data to the list of the inline attachments
	//
	// @param	inData - pointer to the inline data
	// @param	inDataSize - size of the inline data
	// @param	inFileName - pointer to the name of the inline data
	// @param	inContentId - pointer to the inline content identifier
	*/
	void								AddInlineFromData(
											const void* inData,
											EmaiUint32 inDataSize,
											EmaiConstUniCharPtr inFileName,
											EmaiConstUniCharPtr inContentId /*=NULL*/);

	/*!
	// @brief	Add the inline attachment from file to the list of the inline attachments
	//
	// @param	inFileName - pointer to the name of the file
	// @param	inFileFullPath - pointer to the file full path
	// @param	inContentId - pointer to the inline content identifier
	*/
	void								AddInlineFromFile(
											EmaiConstUniCharPtr inFileName,
											EmaiConstUniCharPtr inFileFullPath,
											EmaiConstUniCharPtr inContentId /*=NULL*/);

	/*!
	// @brief	Composes the message
	//
	// @param	inFromContact - pointer to vector of contact fields "From"
	// @param	inSenderContact - pointer to sender contact field
	// @param	inToContact - pointer to vector of contact fields "To"
	// @param	inCCContact - pointer to vector of contact fields "Cc"
	// @param	inBCContact - pointer to vector of contact fields "Bcc"
	// @param	inMessageSubject - pointer to message subject
	*/
	Emai::Message						ComposeMessage(
											Emai::ContactList inFromContact,
											Emai::ContactList inSenderContact,
											Emai::ContactList inToContact,
											Emai::ContactList inCCContact,
											Emai::ContactList inBCContact,
											EmaiConstUniCharPtr inMessageSubject);

};

#endif /*__MESSAGECOMPOSER_HPP__*/
