/*!
// @file		EmaiMessage.hpp
//
// @brief		Defines the set of classes, used for composing and parsing MIME messages
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIMESSAGE_HPP__
#define __EMAIMESSAGE_HPP__

#include <EmaiMessage.h>
#include <EmaiObject.hpp>
#include <EmaiExceptions.hpp>
#include <string>

/*!
// @namespace	Emai
// @brief		Mailit for C/C++ library namespace
// @author		(C) 2006 Dacons LLP.
*/
namespace Emai
{

typedef std::basic_string<EmaiChar>		String;
typedef std::basic_string<EmaiUniChar>	UniString;

/*!
// @class	StringList
// @brief	EmaiStringList wrapper class.\n
//			EmaiStringList is an object which simplifies operations with lists
//			of unicode strings.
//
// @see		EmaiStringList
*/
class StringList : public Object<EmaiStringList>
{
protected:

public:

										StringList(
												EmaiOptions inOptions = EmaiNullOptions);

										StringList(
												EmaiStringList inStringList,
												bool inAddRef);

										StringList(
												const StringList& inCopy);

	StringList&							operator = (
												const StringList& inCopy);


	void								AddString(
												EmaiConstUniCharPtr inString,
												EmaiOptions inOptions = EmaiNullOptions);

	EmaiUint32							GetCount(
												EmaiOptions inOptions = EmaiNullOptions);

	EmaiConstUniCharPtr					GetString(
												EmaiUint32 inIndex,
												EmaiOptions inOptions = EmaiNullOptions);

};

/*!
// @class	ContactList
// @brief	EmaiContactList wrapper class.\n
//			ContactList is an object which simplifies operations with contacts
//			(From, To, Cc etc) in message object
//
// @see		EmaiContactList
*/
class ContactList : public Object<EmaiContactList>
{
protected:

public:

										ContactList(
												EmaiOptions inOptions = EmaiNullOptions);

										ContactList(
												EmaiContactList inContactList,
												bool inAddRef);

										ContactList(
												const ContactList& inCopy);

	ContactList&						operator = (
												const ContactList& inCopy);


	/*!
	// @brief	Adds a contact to the contact list.
	//
	// @param	inName - friendly contact name.\n
	//				The parameter inName can also be used to pass a
	//				combined value (name and address) in the format
	//				of "name" <address> to the plug-in. In this case
	//				the inAddress parameter should be empty.
	// @param	inAddress - e-mail address of the contact
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiContactListAddContact
	*/
	void								AddContact(
												EmaiConstUniCharPtr inName,
												EmaiConstUniCharPtr inAddress,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Retrieves the number of records in contact list.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	On successful exit contains number of the records in the list
	//
	// @see		EmaiContactListGetCount
	*/
	EmaiUint32							GetCount(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Extracts friendly contact name from the contact list, at the specified index.
	//
	// @param	inIndex - zero-based index of the contact in the list
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	Zero-terminated unicode string which contains friendly contact name.
	//				The function returns NULL if invalid parameters are specified
	//				(i.e. inContactList is not valid list or inIndex is out of range)
	//
	// @see		EmaiContactListGetContactName
	*/
	EmaiConstUniCharPtr					GetName(
												EmaiUint32 inIndex,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Extracts e-mail address from the contact list, at the specified index
	//
	// @param	inIndex - zero-based index of the contact in the list
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	Zero-terminated unicode string which contains contact's e-mail address.
	//				The function returns NULL if invalid parameters are specified
	//				(i.e. inContactList is not valid list or inIndex is out of range)
	//
	// @see		EmaiContactListGetContactAddress
	*/
	EmaiConstUniCharPtr					GetAddress(
												EmaiUint32 inIndex,
												EmaiOptions inOptions = EmaiNullOptions);
};

/*!
// @class	ContentField
// @brief	EmaiContentField wrapper class.\n
//			ContentField object can be used to setup or retrieve parsed field data
//				for Content-Type and Content-Disposition fields.
//			For Content-Type field value member contains media type of the message part
//				("text/plain", "application/octet-stream", etc)
//			For Content-Disposition field value member contains disposition of the
//				message part ("attachment", "inline" etc)
//
// @see		EmaiContentField
*/
class ContentField : public Object<EmaiContentField>
{
protected:

public:

										ContentField(
												EmaiConstUniCharPtr inValue,
												EmaiOptions inOptions = EmaiNullOptions);

										ContentField(
												EmaiContentField inContentField,
												bool inAddRef);

										ContentField(
												const ContentField& inCopy);

	ContentField&						operator = (
												const ContentField& inCopy);


	/*!
	// @brief	Adds parameter name and parameter value to the field object.
	//
	// @param	inParamName - parameter name to be added
	//							(for example: "filename")
	// @param	inParamValue - parameter value to be added
	//							(for example: "my file.txt")
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiContentFieldAddParameter
	*/
	void								AddParameter(
												EmaiConstUniCharPtr inParamName,
												EmaiConstUniCharPtr inParamValue,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Extracts the value from the content field.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	Zero-terminated unicode string which contains value of the field.
	//				The function returns NULL if invalid parameters are specified
	//				(i.e. inContentField is not valid object)
	//
	// @see		EmaiContentFieldGetValue
	*/
	EmaiConstUniCharPtr					GetValue(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Retrieves the number of parameters at the content field.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	On successful exit contains the number of the paramets 
	//						at the content field
	//
	// @see		EmaiContentFieldGetParameterCount
	*/
	EmaiUint32							GetParameterCount(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Extracts parameter name at the specified index, from the content field.
	//
	// @param	inIndex - zero-based index of the parameter in the field
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	Zero-terminated unicode string which contains parameter name.
	//				The function returns NULL if invalid parameters are specified
	//				(i.e. inContentField is not valid object or inIndex is out of range)
	//
	// @see		EmaiContentFieldGetParameterName
	*/
	EmaiConstUniCharPtr					GetParameterName(
												EmaiUint32 inIndex,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Extracts parameter value at the specified index, from the content field.
	//
	// @param	inIndex - zero-based index of the parameter in the field
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	Zero-terminated unicode string which contains parameter value.
	//				The function returns NULL if invalid parameters are specified
	//				(i.e. inContentField is not valid object or inIndex is out of range)
	//
	// @see		EmaiContentFieldGetParameterValue
	*/
	EmaiConstUniCharPtr					GetParameterValue(
												EmaiUint32 inIndex,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Finds parameter value with specified name, from the content field.
	//
	// @param	inParamName - parameter name to be found
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	Zero-terminated unicode string which contains parameter value.
	//				The function returns NULL if invalid parameters are specified
	//				(i.e. inContentField is not valid object or parameter with the
	//				specified name is not found in the field)
	//
	// @see		EmaiContentFieldFindParameterValue
	*/
	EmaiConstUniCharPtr					FindParameterValue(
												EmaiConstUniCharPtr inParamName,
												EmaiOptions inOptions = EmaiNullOptions);
};

/*!
// @class	MessagePart
// @brief	EmaiMessagePart wrapper class.\n
//			MessagePart represents message part of the RFC822 message.
//
// @see		EmaiMessagePart
*/
class MessagePart : public Object<EmaiMessagePart>
{
protected:

										MessagePart();//no implementation

public:
	
										~MessagePart();
	
	/*!
	// @brief	Constructor
	*/
										MessagePart(
												const EmaiPartData& inData,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Constructor
	*/
										MessagePart(
												EmaiMessagePart inMessagePart,
												bool inAddRef);

	/*!
	// @brief	Constructor
	*/
										MessagePart(
												const MessagePart& inCopy);

	MessagePart&						operator = (
												const MessagePart& inCopy);
										
	/*!
	// @brief	Appends subparts to the composite part, created via EmaiMessagePartCreateComposite.
	//
	// @param	inSubParts - an array of message parts to be added to the composite part
	// @param	inSubPartCount - number of items in inSubPart array
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiMessagePartAddSubParts
	*/
	void								AddSubParts(
												const EmaiMessagePart inSubParts[],
												EmaiUint32 inSubPartCount,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Retrieves subparts from composite message part.
	//
	// @param	outSubParts - if non-NULL on successful function exit this array
	//					is filled with subparts, otherwise ioSubPartCount
	//					will contain number of subparts
	// @param	ioSubPartCount - if outSubParts is non-NULL, the number of items at the
	//					outSubParts array is passed in function and number of actually
	//					written items is returned on exit. If outSubParts is NULL, 
	//					input value of the parameter is ignored, and number of
	//					subparts is returned on function exit
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiMessagePartGetSubParts
	*/
	void								GetSubParts(
												EmaiMessagePart outSubParts[],
												EmaiUint32* ioSubPartCount,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Retrieves the data from the message part.
	//
	// @param	outData - on successful function exit is filled with
	//					message part data, this data is stored internally in the
	//					message part object - you should not modify of release it
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiMessagePartGetData
	*/
	void								GetData(
												EmaiPartData& outData,
												EmaiOptions inOptions = EmaiNullOptions);
};

/*!
// @class	Message
// @brief	EmaiMessage wrapper class.\n
//			EmaiMessage represents message object in RFC822 format.
//
// @see		Emai::EmaiMessage
*/
class Message : public Object<EmaiMessage>
{
protected:

										Message();//no implementation

public:

										~Message();

	/*!
	// @brief	Constructor
	*/									
										Message(
												MessagePart& inRootPart,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Constructor
	*/
										Message(
												const void* inData,
												EmaiUint32 inDataSize,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Constructor
	*/
										Message(
												EmaiMessage inMessage,
												bool inAddRef);
	/*!
	// @brief	Constructor
	*/
										Message(
												const Message& inCopy);
	Message&							operator = (
												const Message& inCopy);
										
	void								SetContactList(
												const Emai::ContactList& inContactList,
												EmaiOptions inOptions);

	/*!
	// @brief	Adds/replaces field to/at the message header.
	//
	// @param	inFieldName - field name; if inFieldName is non-NULL, then 
	//							inOptions parameter should be EmaiNullOptions;
	/							if inFieldName is NULL, then  inOptions parameter should
	//							specify one of the contact fields (EmaiFrom, EmaiTo, etc)
	// @param	inFieldValue - field value, if NULL - the field is removed from the
	//							message header
	// @param	inOptions - EmaiNullOptions, or EmaiFrom, EmaiTo, etc
	//
	// @see		EmaiMessageSetField
	*/
	void								SetField(
												EmaiConstUniCharPtr inFieldName,
												EmaiConstUniCharPtr inFieldValue,
												EmaiOptions inOptions = EmaiNullOptions);

	/*
	// @brief	Copies contact field from the message header.
	//
	// @param	inOptions - EmaiFrom, EmaiTo, etc
	//
	// @return	On exit will be filled with copy of the contact list
	//					from the message, any modifications of the returned
	//					contact list doesn't affect message contacts
	//
	// @see		EmaiMessageCopyContactList
	*/
	ContactList							CopyContactList(
												EmaiOptions inOptions);
	/*!
	// @brief	Retrieves field content
	//
	// @param	inFieldName - field name which content should be retrieved
	// @param	inOptions - EmaiFailOnUnknownCharset or EmaiNullOptions
	//
	// @return	Zero-terminated unicode string which contains field value.
	//				The function returns NULL if invalid parameters are specified
	//				(i.e. inFieldName is not valid object or parameter with the
	//				specified name is not found in the field)
	//
	// @see		EmaiMessageGetFieldByName
	*/
	EmaiConstUniCharPtr					GetField(
												EmaiConstUniCharPtr inFieldName,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Retrieves the root message part from message object.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @return	On successful exit contains root message part object
	//
	// @see		EmaiMessageGetRootPart
	*/	
	MessagePart							GetRootPart(
												EmaiOptions inOptions = EmaiNullOptions);

	void								GetData(
												const void** outData,		/*!< can be NULL*/
												EmaiUint32* outHeaderSize,	/*!< can be NULL*/
												EmaiUint32* outDataSize,	/*!< can be NULL*/
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Retrieves message received date and time.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	// @param	inFormatString - printf-like format string
	//
	// @return	On successful exit contains received date and time
	//
	// @see		EmaiMessageGetReceivedDateTime
	*/	
	void									GetReceivedDateTime(
												tm* outDateTime,
												EmaiOptions inOptions = EmaiNullOptions);
};


/*
// StringList class implemetation
*/
//constructors
inline
StringList::StringList(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiStringListCreate(&mObject, inOptions));
}

inline
StringList::StringList(
		EmaiStringList inStringList,
		bool inAddRef
	):
	Parent(inStringList, inAddRef)
{
}

inline
StringList::StringList(
		const StringList& inCopy
	):
	Parent(inCopy)
{
}

inline StringList&
StringList::operator = (
		const StringList& inCopy
	)
{
	Parent::operator = (inCopy);
	return *this;
}

inline void
StringList::AddString(
		EmaiConstUniCharPtr inString,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiStringListAddString(mObject, inString, inOptions));
}

inline EmaiUint32
StringList::GetCount(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiUint32 count = 0;
	CheckErrorCode(EmaiStringListGetCount(mObject, &count, inOptions));
	return count;
}

inline EmaiConstUniCharPtr
StringList::GetString(
		EmaiUint32 inIndex,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiConstUniCharPtr name = EmaiStringListGetString(mObject,
								inIndex, inOptions);
	if (name == NULL)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return name;
}


/*
// ContactList class implemetation
*/
//constructors
inline
ContactList::ContactList(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiContactListCreate(&mObject, inOptions));
}

inline
ContactList::ContactList(
		EmaiContactList inContactList,
		bool inAddRef
	):
	Parent(inContactList, inAddRef)
{
}

inline
ContactList::ContactList(
		const ContactList& inCopy
	):
	Parent(inCopy)
{
}

inline ContactList&
ContactList::operator = (
		const ContactList& inCopy
	)
{
	Parent::operator = (inCopy);
	return *this;
}

inline void
ContactList::AddContact(
		EmaiConstUniCharPtr inName,
		EmaiConstUniCharPtr inAddress,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiContactListAddContact(mObject, inName, inAddress,
							NULL, inOptions));
}

inline EmaiUint32
ContactList::GetCount(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiUint32 count = 0;
	CheckErrorCode(EmaiContactListGetCount(mObject, &count, inOptions));
	return count;
}

inline EmaiConstUniCharPtr
ContactList::GetName(
		EmaiUint32 inIndex,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiConstUniCharPtr name = EmaiContactListGetContactName(mObject,
								inIndex, inOptions);
	if (name == NULL)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return name;
}

inline EmaiConstUniCharPtr
ContactList::GetAddress(
		EmaiUint32 inIndex,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiConstUniCharPtr address = EmaiContactListGetContactAddress(mObject,
								inIndex, inOptions);
	if (address == NULL)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return address;
}

/*
// ContentField class implemetation
*/
//constructors
inline
ContentField::ContentField(
		EmaiConstUniCharPtr inValue,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiContentFieldCreate(&mObject, inValue, inOptions));
}

inline
ContentField::ContentField(
		EmaiContentField inContentField,
		bool inAddRef
	):
	Parent(inContentField, inAddRef)
{
}

inline
ContentField::ContentField(
		const ContentField& inCopy
	):
	Parent(inCopy)
{
}

inline ContentField&
ContentField::operator = (
		const ContentField& inCopy
	)
{
	Parent::operator = (inCopy);
	return *this;
}

inline void
ContentField::AddParameter(
		EmaiConstUniCharPtr inParamName,
		EmaiConstUniCharPtr inParamValue,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiContentFieldAddParameter(mObject, inParamName,
							inParamValue, inOptions));
}

inline EmaiConstUniCharPtr
ContentField::GetValue(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiConstUniCharPtr value = EmaiContentFieldGetValue(mObject, inOptions);
	if (value == NULL)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return value;
}

inline EmaiUint32
ContentField::GetParameterCount(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiUint32 count = 0;
	CheckErrorCode(EmaiContentFieldGetParameterCount(mObject, &count, inOptions));
	return count;
}

inline EmaiConstUniCharPtr
ContentField::GetParameterName(
		EmaiUint32 inIndex,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiConstUniCharPtr name = EmaiContentFieldGetParameterName(mObject,
								inIndex, inOptions);
	if (name == NULL)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return name;
}

inline EmaiConstUniCharPtr
ContentField::GetParameterValue(
		EmaiUint32 inIndex,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiConstUniCharPtr value = EmaiContentFieldGetParameterValue(mObject,
								inIndex, inOptions);
	if (value == NULL)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return value;
}

inline EmaiConstUniCharPtr
ContentField::FindParameterValue(
		EmaiConstUniCharPtr inParamName,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	return EmaiContentFieldFindParameterValue(mObject,
								inParamName, inOptions);
}


/*
// MessagePart class implemetation
*/
//constructors
inline
MessagePart::MessagePart(
		const EmaiPartData& inData,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiMessagePartCreate(&mObject,
						&inData, inOptions));
}

inline
MessagePart::MessagePart(
		EmaiMessagePart inMessagePart,
		bool inAddRef
	):
	Parent(inMessagePart, inAddRef)
{
}

inline
MessagePart::MessagePart(
		const MessagePart& inCopy
	):
	Parent(inCopy)
{
}

inline
MessagePart::~MessagePart()
{
}

inline MessagePart&
MessagePart::operator = (
		const MessagePart& inCopy
	)
{
	Parent::operator = (inCopy);
	return *this;
}

inline void
MessagePart::AddSubParts(
		const EmaiMessagePart inSubPart[],
		EmaiUint32 inSubPartCount,
		EmaiOptions inOptions /*= EmaiNullOptions*/)
{
	CheckErrorCode(EmaiMessagePartAddSubParts(
				mObject, inSubPart, inSubPartCount, inOptions));
}

inline void
MessagePart::GetSubParts(
		EmaiMessagePart outSubParts[],
		EmaiUint32* ioSubPartCount,
		EmaiOptions inOptions /*= EmaiNullOptions*/)
{
	CheckErrorCode(EmaiMessagePartGetSubParts(
				mObject, outSubParts, ioSubPartCount, inOptions));
}

inline void
MessagePart::GetData(
		EmaiPartData& outData,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiMessagePartGetData(
				mObject, &outData, inOptions));
}


/*
// Message class implemetation
*/
inline
Message::Message(
		MessagePart& inRootPart,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiMessageCreate(&mObject, inRootPart, inOptions));
}

inline
Message::Message(
		const void* inData,
		EmaiUint32 inDataSize,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiMessageCreateFromData(
				&mObject, inData, inDataSize, inOptions));
}

inline
Message::Message(
		EmaiMessage inMessage,
		bool inAddRef
	):
	Parent(inMessage, inAddRef)
{
}

inline
Message::Message(
		const Message& inCopy
	):
	Parent(inCopy)
{
}

inline
Message::~Message()
{
}

inline Message&
Message::operator = (
		const Message& inCopy
	)
{
	Parent::operator = (inCopy);
	return *this;
}

inline void
Message::SetContactList(
		const Emai::ContactList& inContactList,
		EmaiOptions inOptions
	)
{
	CheckErrorCode(EmaiMessageSetContactList(mObject, inContactList, inOptions));
}

inline void
Message::SetField(
		EmaiConstUniCharPtr inFieldName,
		EmaiConstUniCharPtr inFieldValue,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiMessageSetField(mObject,
					inFieldName, inFieldValue, inOptions));
}

inline ContactList
Message::CopyContactList(
		EmaiOptions inOptions
	)
{
	EmaiContactList list = NULL;
	EmaiError error = EmaiMessageCopyContactList(mObject, &list, inOptions);
	if (error == EmaiErrNoSuchField)
		list = NULL;
	else
		CheckErrorCode(error);

	return ContactList(list, false);
}

inline EmaiConstUniCharPtr
Message::GetField(
		EmaiConstUniCharPtr inFieldName,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiConstUniCharPtr fieldValue = NULL;
	EmaiError error = EmaiMessageGetFieldByName(mObject,
					inFieldName, &fieldValue, inOptions);
	if (error == EmaiErrNoSuchField)
		fieldValue = NULL;
	else
		CheckErrorCode(error);

	return fieldValue;
}

inline MessagePart
Message::GetRootPart(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiMessagePart rootPart = NULL;
	CheckErrorCode(EmaiMessageGetRootPart(mObject, &rootPart, inOptions));
	return MessagePart(rootPart, true);
}

inline void
Message::GetData(
		const void** outData,		/*!< can be NULL*/
		EmaiUint32* outHeaderSize,	/*!< can be NULL*/
		EmaiUint32* outDataSize,	/*!< can be NULL*/
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiMessageGetData(mObject, outData, outHeaderSize,
				outDataSize, inOptions));
}

inline void
Message::GetReceivedDateTime(
		tm* outDateTime,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiMessageGetReceivedDateTime(mObject, outDateTime, inOptions));
}

}
#endif /*__EMAIMESSAGE_H__*/
